function [RANDF_scale CohRandf InCohRandf]=Setup_disarray(GAUSS_KRNL_SPECT,COHERENCE,REACH_SCALE,CohRandf,InCohRandf)
%
% USAGE:   [RANDF_scale CohRandf InCohRandf]=Setup_disarray(GAUSS_KRNL_SPECT,COHERENCE,REACH_SCALE,CohRandf,InCohRandf)

% Sets up random displacement fields (for x and y coordinates) for each scale.
% Starting points are gaussian random fields (NxMx2, two layers being for x and y) which are low-passed with thesame kernels that were used to low-pass and decompose the image.
% The coherent random field is defined once and used for all scales, whereas an incoherent field is defined independently for each scale.
% The values of lowpass
%
% INPUTS:
%
% GAUSS_KRNL_SPECT: cell array containing spectra of the Gaussian kernels at each scale (finest to coarsest)
% COHERENCE: Scalar comprised between 0 (disarray is completely incoherent between scales) and 1 (the same noise field is used to disarray all scales).
%     The number indicates the ratio of coherent to incoherent noise in the displacement field.
% REACH_SCALE: vector containing reach (maximum displacement in pixels) to be applied to each scale
% CohRandf: Matrix containing the coherent random displacement field for x
% and y (NxMx2).
% InCohRandf: Cell array containing random displacement fields for each
% scale AFTER applying filtering
%
% OUTPUTS:
%
% RANDF_scale: cell array containing random displacement fields for each scale (each cell one scale, finest to coarsest. Each field NxMx2, two layers being for x and y)
% CohRandf: Matrix containing the coherent random displacement field for x
% and y (NxMx2).
% InCohRandf: Cell array containing random displacement fields for each
% scale AFTER applying filtering

imsiz=size(GAUSS_KRNL_SPECT{1});

% coherent random field to be used in all scales

if ((nargin>3)&(~isempty(CohRandf)))
    CohRANDF_x=CohRandf(:,:,1);
    CohRANDF_y=CohRandf(:,:,2);
else
    CohRANDF_x=fft2(randn(imsiz(1),imsiz(2)));
    CohRANDF_y=fft2(randn(imsiz(1),imsiz(2)));
    if nargout>1
        CohRandf(:,:,1)=CohRANDF_x;
        CohRandf(:,:,2)=CohRANDF_y;
    end
end

for nscale=1:length(GAUSS_KRNL_SPECT)
    
    % filter coherent random field
    CohRANDF_scale(:,:,1)=real(ifft2(CohRANDF_x.*GAUSS_KRNL_SPECT{nscale}));
    CohRANDF_scale(:,:,2)=real(ifft2(CohRANDF_y.*GAUSS_KRNL_SPECT{nscale}));
    CohRANDF_scale=CohRANDF_scale./std(CohRANDF_scale(:));
    % normalize to std=1
    
    if ((nargin>4)&(~isempty(InCohRandf))) % scale specific incoherent random field was provided        
        inCohRANDF_x=InCohRandf{nscale}(:,:,1);
        inCohRANDF_y=InCohRandf{nscale}(:,:,2);
    else
        % incoherent random field scale specific
        inCohRANDF_x=fft2(randn(imsiz(1),imsiz(2)));
        inCohRANDF_y=fft2(randn(imsiz(1),imsiz(2)));
        
    end
    
    % filter incoherent random field
    inCohRANDF_scale(:,:,1)=real(ifft2(inCohRANDF_x.*GAUSS_KRNL_SPECT{nscale}));
    inCohRANDF_scale(:,:,2)=real(ifft2(inCohRANDF_y.*GAUSS_KRNL_SPECT{nscale}));
    inCohRANDF_scale=inCohRANDF_scale./std(inCohRANDF_scale(:));
    % normalize to std=1
    if nargout>2
        InCohRandf{nscale}(:,:,1)=inCohRANDF_x;
        InCohRandf{nscale}(:,:,2)=inCohRANDF_y;
    end
    
    % Add coherent and incoherent in parts defined by COHERENCE
    RANDF_scale{nscale}=CohRANDF_scale*COHERENCE+inCohRANDF_scale*(1-COHERENCE);
    % Multiply by reach value at the appropriate scale
    RANDF_scale{nscale}=RANDF_scale{nscale}*REACH_SCALE(nscale);
end